
#include <stdio.h>
#include <stdlib.h>
#include "cell.h"
#include "csi.h"

extern void	Error(char *msg),
		RangeList(char *rl,size_t lim,char *pv),
		ReadSliceImage(char *path,int Nx,int Ny,Cell **cell);

typedef struct {
		Cell	*T;
		size_t	*X,*Y,*Z;
	} Struct;

static Struct	*Alloc(int M,Cell *T,Cell c)
{
	Struct	*s;

	if ((s=(Struct *)malloc(sizeof(Struct)))==NULL ||
	    (s->X=(size_t *)calloc(M,sizeof(size_t)))==NULL ||
	    (s->Y=(size_t *)calloc(M,sizeof(size_t)))==NULL ||
	    (s->Z=(size_t *)calloc(M,sizeof(size_t)))==NULL)
	    Error("count memory allocation error.");

	s->T=T; s->X-=c; s->Y-=c; s->Z-=c; return s;
}

static void	Print(int i,int j,size_t X,size_t Y,size_t Z)
{
	if (X>0 || Y>0 || Z>0)
	(void)printf("%d\t%d\t%lu\t%lu\t%lu\n",
		     i,j,(unsigned long)X,(unsigned long)Y,(unsigned long)Z);
}

#define PV	(size_t)((Cell)(~0))

int	main(int argc,char **argv)
{
	char	**path,pv1[PV+1],pv2[PV+1];
	int	Nx,Ny,Nz,bps,y,N,i,j,k,z,x;
	Cell	**cell,**last,T1[PV+1],T2[PV+1],a,b;
	Struct	*S[PV+1],*s0,*s;

	if (argc!=5)
	    Error("usage : si_mfc directory nameFile rangeList1 rangeList2");

	path=CheckSliceImages(argv[1],argv[2],&Nx,&Ny,&Nz,&bps);

	RangeList(argv[3],PV,pv1);
	RangeList(argv[4],PV,pv2);

	if ((cell=(Cell **)malloc(sizeof(Cell *)*Ny))==NULL ||
	    (*cell=(Cell *)malloc(sizeof(Cell)*Ny*Nx))==NULL ||
	    (last=(Cell **)malloc(sizeof(Cell *)*(Ny+1)))==NULL ||
	    (*last=(Cell *)calloc((size_t)(Nx+1)*(Ny+1),sizeof(Cell)))==NULL)
	    Error("image memory allocation error.");

	for (y=1; y<Ny; y++) cell[y]=cell[y-1]+Nx;

	++last; ++last[-1]; for (y=0; y<Ny; y++) last[y]=last[y-1]+Nx+1;

	N=1<<bps; T1[0]=T2[0]=0; i=j=1;
	for (k=1; k<N; k++) {
	    S[k]=NULL; T1[k]=(pv1[k])?(Cell)i++:0;
		       T2[k]=(pv2[k])?(Cell)j++:0;
	}
	s0=S[0]=(i>=j)?Alloc(i,T1,0):
		       Alloc(j,T2,0); a=b=1; --i; --j;
	while (i>0 && j>0) {
	    while (T1[a]==0) ++a;
	    while (T2[b]==0) ++b;
	    if (a<=b) {
		S[a]=Alloc(j,T2,T2[b]); if (a==b) { ++b; --j; }
					++a; --i;
	    }
	    else {
		S[b]=Alloc(i,T1,T1[a]); ++b; --j;
	    }
	}

#define COUNT(L,XYZ)	\
	if (a<=(b=L)) if (b==0) ++s0->XYZ[0]; else { \
	    if ((s=S[a])!=NULL && (k=s->T[b])>0) ++s->XYZ[k]; \
	} \
	else \
	    if ((s=S[b])!=NULL && (k=s->T[a])>0) ++s->XYZ[k]

#define COUNT_EOI(L,XYZ)	\
	if ((b=L)==0) ++s0->XYZ[0]; else if ((k=s0->T[b])>0) ++s0->XYZ[k]

	for (z=0; z<Nz; z++) {
	    ReadSliceImage(path[z],Nx,Ny,cell);

	    for (y=0; y<Ny; y++) {
		for (x=0; x<Nx; x++) {
		    a=cell[y][x]; COUNT(last[y  ][x-1],X);
				  COUNT(last[y-1][x  ],Y);
				  COUNT(last[y  ][x  ],Z); last[y][x]=a;
		}
		COUNT_EOI(last[y][Nx-1],X);
	    }
	    for (x=0; x<Nx; x++) COUNT_EOI(last[Ny-1][x],Y);
	}
	for (y=0; y<Ny; y++) for (x=0; x<Nx; x++) COUNT_EOI(last[y][x],Z);

	if (pv1[0] && pv2[0]) Print(0,0,s0->X[0],s0->Y[0],s0->Z[0]);

	for (j=1; j<N; j++)
	    if ((k=s0->T[j])>0) Print(0,j,s0->X[k],s0->Y[k],s0->Z[k]);

	for (i=1; i<N; i++)
	    if ((s=S[i])!=NULL)
		for (j=i; j<N; j++)
		    if ((k=s->T[j])>0) Print(i,j,s->X[k],s->Y[k],s->Z[k]);

	return 0;
}
