
#include <stdio.h>
#include <stdlib.h>
#include "cell.h"
#include "rif.h"

extern void	Error(char *msg),
		RangeList(char *rl,size_t lim,char *pv);

typedef struct {
		Cell	*T;
		size_t	*X,*Y;
	} Struct;

static Struct	*Alloc(int M,Cell *T,Cell c)
{
	Struct	*s;

	if ((s=(Struct *)malloc(sizeof(Struct)))==NULL ||
	    (s->X=(size_t *)calloc(M,sizeof(size_t)))==NULL ||
	    (s->Y=(size_t *)calloc(M,sizeof(size_t)))==NULL)
	    Error("count memory allocation error.");

	s->T=T; s->X-=c; s->Y-=c; return s;
}

static void	Print(int i,int j,size_t X,size_t Y)
{
	if (X>0 || Y>0) (void)printf("%d\t%d\t%lu\t%lu\n",
				     i,j,(unsigned long)X,(unsigned long)Y);
}

#define PV	(size_t)((Cell)(~0))

int	main(int argc,char **argv)
{
	int	Nx,Ny,bps,N,i,j,k,y,x;
	Cell	**cell,*last,T1[PV+1],T2[PV+1],a,b;
	char	pv1[PV+1],pv2[PV+1];
	Struct	*S[PV+1],*s0,*s;

	if (argc!=4) Error("usage : t_mec TIFF rangeList1 rangeList2");

	ReadImageFile(argv[1],&Nx,&Ny,&bps,&cell,NULL);

	RangeList(argv[2],PV,pv1);
	RangeList(argv[3],PV,pv2);

	if ((last=(Cell *)calloc(Nx+1,sizeof(Cell)))==NULL)
	    Error("image memory allocation error.");

	++last;

	N=1<<bps; T1[0]=T2[0]=0; i=j=1;
	for (k=1; k<N; k++) {
	    S[k]=NULL; T1[k]=(pv1[k])?(Cell)i++:0;
		       T2[k]=(pv2[k])?(Cell)j++:0;
	}
	s0=S[0]=(i>=j)?Alloc(i,T1,0):
		       Alloc(j,T2,0); a=b=1; --i; --j;
	while (i>0 && j>0) {
	    while (T1[a]==0) ++a;
	    while (T2[b]==0) ++b;
	    if (a<=b) {
		S[a]=Alloc(j,T2,T2[b]); if (a==b) { ++b; --j; }
					++a; --i;
	    }
	    else {
		S[b]=Alloc(i,T1,T1[a]); ++b; --j;
	    }
	}

#define COUNT(L,XY)	\
	if (a<=(b=L)) if (b==0) ++s0->XY[0]; else { \
	    if ((s=S[a])!=NULL && (k=s->T[b])>0) ++s->XY[k]; \
	} \
	else \
	    if ((s=S[b])!=NULL && (k=s->T[a])>0) ++s->XY[k]

#define COUNT_EOI(L,XY)	\
	if ((b=L)==0) ++s0->XY[0]; else if ((k=s0->T[b])>0) ++s0->XY[k]

	for (y=0; y<Ny; y++) {
	    for (x=0; x<Nx; x++) {
		a=cell[y][x]; COUNT(last[x-1],X); COUNT(last[x],Y); last[x]=a;
	    }
	    COUNT_EOI(last[Nx-1],X);
	}
	for (x=0; x<Nx; x++) COUNT_EOI(last[x],Y);

	if (pv1[0] && pv2[0]) Print(0,0,s0->X[0],s0->Y[0]);

	for (j=1; j<N; j++)
	    if ((k=s0->T[j])>0) Print(0,j,s0->X[k],s0->Y[k]);

	for (i=1; i<N; i++)
	    if ((s=S[i])!=NULL)
		for (j=i; j<N; j++)
		    if ((k=s->T[j])>0) Print(i,j,s->X[k],s->Y[k]);

	return 0;
}
